<?php
namespace Zeedhi\Tools\Generator;

use Doctrine\DBAL\DriverManager;

class ConnectionConfig {

    /** @var string */
    protected $driver;
    /** @var string */
    protected $host;
    /** @var string */
    protected $port;
    /** @var string */
    protected $username;
    /** @var string */
    protected $password;
    /** @var string */
    protected $dbname;
    /** @var bool */
    protected $service;
    /** @var bool */
    protected $isOracleWallet = false;

    /**
     * Constructor...
     *
     * @param string $driver
     * @param string $host
     * @param string $port
     * @param string $username
     * @param string $password
     * @param string $dbname
     * @param bool   $service
     */
    public function __construct($driver, $host, $port, $username, $password, $dbname, $service, $isEncrypted) {
        $this->driver   = $driver;
        $this->host     = $host;
        $this->port     = $port;
        $this->username = $username;
        $this->password = $password;
        $this->dbname = $dbname;
        $this->service = $service;
        $this->isEncrypted = $isEncrypted;
    }


    public static function factoryFromArray($configArray) {
        $instance = new static(
            $configArray['driver'],
            isset($configArray['host']) ? $configArray['host'] : '',
            isset($configArray['port']) ? $configArray['port'] : '',
            isset($configArray['username']) ? $configArray['username'] : $configArray['user'],
            $configArray['password'],
            $configArray['dbname'],
            isset($configArray['service']) ? $configArray['service'] : false,
            isset($configArray['isEncrypted']) ? $configArray['isEncrypted'] : false
        );

        if(isset($configArray['isOracleWallet'])) {
            $instance->isOracleWallet = $configArray['isOracleWallet'];
        }

        return $instance;
    }

    /**
     * @param string $salt
     * @return string
     */
    protected static function padSalt($salt) {
        return str_pad($salt, 32, "\0");
    }

    public static function decryptPassword($password, $salt = "zeedhi") {
        return trim(
            mcrypt_decrypt(
                MCRYPT_RIJNDAEL_256,
                self::padSalt($salt),
                base64_decode($password),
                MCRYPT_MODE_ECB,
                mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)
            )
        );
    }

    public static function encryptPassword($password, $salt = "zeedhi") {
        return trim(
            base64_encode(
                mcrypt_encrypt(
                    MCRYPT_RIJNDAEL_256,
                    self::padSalt($salt),
                    $password,
                    MCRYPT_MODE_ECB,
                    mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)
                )
            )
        );
    }

    /**
     * @return \Doctrine\DBAL\Connection
     */
    public function factoryConnection() {
        if ($this->isOracleWallet) {
            $connection = DriverManager::getConnection(array(
                "driver"      => $this->driver,
                "user"        => '/',
                "password"    => '',
                "dbname"      => $this->dbname,
                "sessionMode" => OCI_CRED_EXT,
                "service"     => $this->service
            ));
        } else {
            $pass = $this->isEncrypted ? self::decryptPassword($this->password) : $this->password;
            $connection = DriverManager::getConnection(array(
                "driver"   => $this->driver,
                "host"     => $this->host,
                "port"     => $this->port,
                "user"     => $this->username,
                "password" => $pass,
                "dbname"   => $this->dbname,
                "service"  => $this->service
            ));
        }
        return $connection;
    }
}